@@ -35,6 +35,10 @@ clients must be made or how a client should react.
 #include <stdio.h>
 #include <poll.h>
 
+#ifndef BUF_SIZE
+#define BUF_SIZE 16384
+#endif
+
 #define SAFE_FREE(x) do { if ((x) != NULL) {free(x); x=NULL;} } while(0)
 
 #ifndef __unused__
@@ -88,7 +92,7 @@ cleanup_push(struct cleanup_node_struct** head_ref,
     // Allocate memory for node
     struct cleanup_node_struct *new_node = malloc(sizeof *new_node);
 
-    if (head_ref != NULL) {
+    if (*head_ref != NULL) {
         new_node->next = *head_ref;
     } else {
         new_node->next = NULL;
@@ -353,7 +357,7 @@ my_fd_data_function(UNUSED_PARAM(socket_t fd),
     ssh_channel channel = event_fd_data->channel;
     ssh_session session;
     int len, i, wr;
-    char buf[16384];
+    char buf[BUF_SIZE];
     int blocking;
 
     if (channel == NULL) {
@@ -418,7 +422,7 @@ my_fd_data_function(UNUSED_PARAM(socket_t fd),
                     break;
                 }
                 wr += i;
-                _ssh_log(SSH_LOG_FUNCTIONS, "=== my_fd_data_function", "channel_write (%d from %d)", wr, len);
+                _ssh_log(SSH_LOG_FUNCTIONS, "=== my_fd_data_function", "ssh_channel_write (%d from %d)", wr, len);
             } while (i > 0 && wr < len);
         }
         else {
@@ -522,6 +526,7 @@ message_callback(UNUSED_PARAM(ssh_session session),
                     SAFE_FREE(pFd);
                     SAFE_FREE(cb_chan);
                     SAFE_FREE(event_fd_data);
+                    close(socket_fd);
                     return 1;
                 }
 
