@@ -92,11 +92,7 @@ int server_set_kex(ssh_session session)
     size_t len;
     int ok;
 
-    /* Skip if already set, for example for the rekey or when we do the guessing
-     * it could have been already used to make some protocol decisions. */
-    if (server->methods[0] != NULL) {
-        return SSH_OK;
-    }
+    ZERO_STRUCTP(server);
 
     ok = ssh_get_random(server->cookie, 16, 0);
     if (!ok) {
@@ -195,13 +191,7 @@ int server_set_kex(ssh_session session)
         }
     }
 
-    /* Do not append the extensions during rekey */
-    if (session->flags & SSH_SESSION_FLAG_AUTHENTICATED) {
-        return SSH_OK;
-    }
-
-    rc = ssh_kex_append_extensions(session, server);
-    return rc;
+    return 0;
 }
 
 int ssh_server_init_kex(ssh_session session) {
@@ -345,121 +335,116 @@ ssh_get_key_params(ssh_session session,
  * @brief A function to be called each time a step has been done in the
  * connection.
  */
-static void ssh_server_connection_callback(ssh_session session)
-{
+static void ssh_server_connection_callback(ssh_session session){
     int rc;
 
-    switch (session->session_state) {
-    case SSH_SESSION_STATE_NONE:
-    case SSH_SESSION_STATE_CONNECTING:
-    case SSH_SESSION_STATE_SOCKET_CONNECTED:
-        break;
-    case SSH_SESSION_STATE_BANNER_RECEIVED:
-        if (session->clientbanner == NULL) {
-            goto error;
-        }
-        set_status(session, 0.4f);
-        SSH_LOG(SSH_LOG_PROTOCOL,
-                "SSH client banner: %s", session->clientbanner);
-
-        /* Here we analyze the different protocols the server allows. */
-        rc = ssh_analyze_banner(session, 1);
-        if (rc < 0) {
-            ssh_set_error(session, SSH_FATAL,
-                          "No version of SSH protocol usable (banner: %s)",
-                          session->clientbanner);
-            goto error;
-        }
-
-        /* from now, the packet layer is handling incoming packets */
-        session->socket_callbacks.data = ssh_packet_socket_callback;
-        ssh_packet_register_socket_callback(session, session->socket);
-
-        ssh_packet_set_default_callbacks(session);
-        set_status(session, 0.5f);
-        session->session_state = SSH_SESSION_STATE_INITIAL_KEX;
-        if (ssh_send_kex(session) < 0) {
-            goto error;
-        }
-        break;
-    case SSH_SESSION_STATE_INITIAL_KEX:
-        /* TODO: This state should disappear in favor of get_key handle */
-        break;
-    case SSH_SESSION_STATE_KEXINIT_RECEIVED:
-        set_status(session, 0.6f);
-        if ((session->flags & SSH_SESSION_FLAG_KEXINIT_SENT) == 0) {
-            if (server_set_kex(session) == SSH_ERROR)
+    switch(session->session_state){
+        case SSH_SESSION_STATE_NONE:
+        case SSH_SESSION_STATE_CONNECTING:
+        case SSH_SESSION_STATE_SOCKET_CONNECTED:
+            break;
+        case SSH_SESSION_STATE_BANNER_RECEIVED:
+            if (session->clientbanner == NULL) {
                 goto error;
-            /* We are in a rekeying, so we need to send the server kex */
-            if (ssh_send_kex(session) < 0)
-                goto error;
-        }
-        ssh_list_kex(&session->next_crypto->client_kex); // log client kex
-        if (ssh_kex_select_methods(session) < 0) {
-            goto error;
-        }
-        if (crypt_set_algorithms_server(session) == SSH_ERROR)
-            goto error;
-        set_status(session, 0.8f);
-        session->session_state = SSH_SESSION_STATE_DH;
-        break;
-    case SSH_SESSION_STATE_DH:
-        if (session->dh_handshake_state == DH_STATE_FINISHED) {
+            }
+            set_status(session, 0.4f);
+            SSH_LOG(SSH_LOG_PROTOCOL,
+                    "SSH client banner: %s", session->clientbanner);
 
-            rc = ssh_packet_set_newkeys(session, SSH_DIRECTION_IN);
-            if (rc != SSH_OK) {
+            /* Here we analyze the different protocols the server allows. */
+            rc = ssh_analyze_banner(session, 1);
+            if (rc < 0) {
+                ssh_set_error(session, SSH_FATAL,
+                        "No version of SSH protocol usable (banner: %s)",
+                        session->clientbanner);
                 goto error;
             }
 
-            /*
-             * If the client supports extension negotiation, we will send
-             * our supported extensions now. This is the first message after
-             * sending NEWKEYS message and after turning on crypto.
-             */
-            if (session->extensions & SSH_EXT_NEGOTIATION &&
-                session->session_state != SSH_SESSION_STATE_AUTHENTICATED) {
+            /* from now, the packet layer is handling incoming packets */
+            ssh_packet_register_socket_callback(session, session->socket);
+
+            ssh_packet_set_default_callbacks(session);
+            set_status(session, 0.5f);
+            session->session_state=SSH_SESSION_STATE_INITIAL_KEX;
+            if (ssh_send_kex(session, 1) < 0) {
+                goto error;
+            }
+            break;
+        case SSH_SESSION_STATE_INITIAL_KEX:
+            /* TODO: This state should disappear in favor of get_key handle */
+            break;
+        case SSH_SESSION_STATE_KEXINIT_RECEIVED:
+            set_status(session,0.6f);
+            if(session->next_crypto->server_kex.methods[0]==NULL){
+                if(server_set_kex(session) == SSH_ERROR)
+                    goto error;
+                /* We are in a rekeying, so we need to send the server kex */
+                if(ssh_send_kex(session, 1) < 0)
+                    goto error;
+            }
+            ssh_list_kex(&session->next_crypto->client_kex); // log client kex
+            if (ssh_kex_select_methods(session) < 0) {
+                goto error;
+            }
+            if (crypt_set_algorithms_server(session) == SSH_ERROR)
+                goto error;
+            set_status(session,0.8f);
+            session->session_state=SSH_SESSION_STATE_DH;
+            break;
+        case SSH_SESSION_STATE_DH:
+            if(session->dh_handshake_state==DH_STATE_FINISHED){
+
+                rc = ssh_packet_set_newkeys(session, SSH_DIRECTION_IN);
+                if (rc != SSH_OK) {
+                    goto error;
+                }
 
                 /*
-                 * Only send an SSH_MSG_EXT_INFO message the first time the
-                 * client undergoes NEWKEYS.  It is unexpected for this message
-                 * to be sent upon rekey, and may cause clients to log error
-                 * messages.
-                 *
-                 * The session_state can not be used for this purpose because it
-                 * is re-set to SSH_SESSION_STATE_KEXINIT_RECEIVED during rekey.
-                 * So, use the connected flag which transitions from non-zero
-                 * below.
-                 *
-                 * See also:
-                 * - https://bugzilla.mindrot.org/show_bug.cgi?id=2929
+                 * If the client supports extension negotiation, we will send
+                 * our supported extensions now. This is the first message after
+                 * sending NEWKEYS message and after turning on crypto.
                  */
-                if (session->connected == 0) {
-                    ssh_server_send_extensions(session);
+                if (session->extensions & SSH_EXT_NEGOTIATION &&
+                    session->session_state != SSH_SESSION_STATE_AUTHENTICATED) {
+
+                    /*
+                     * Only send an SSH_MSG_EXT_INFO message the first time the client
+                     * undergoes NEWKEYS.  It is unexpected for this message to be sent
+                     * upon rekey, and may cause clients to log error messages.
+                     *
+                     * The session_state can not be used for this purpose because it is
+                     * re-set to SSH_SESSION_STATE_KEXINIT_RECEIVED during rekey.  So,
+                     * use the connected flag which transitions from non-zero below.
+                     *
+                     * See also:
+                     * - https://bugzilla.mindrot.org/show_bug.cgi?id=2929
+                     */
+                    if (session->connected == 0) {
+                        ssh_server_send_extensions(session);
+                    }
                 }
+
+                set_status(session,1.0f);
+                session->connected = 1;
+                session->session_state=SSH_SESSION_STATE_AUTHENTICATING;
+                if (session->flags & SSH_SESSION_FLAG_AUTHENTICATED)
+                    session->session_state = SSH_SESSION_STATE_AUTHENTICATED;
+
             }
-
-            set_status(session, 1.0f);
-            session->connected = 1;
-            session->session_state = SSH_SESSION_STATE_AUTHENTICATING;
-            if (session->flags & SSH_SESSION_FLAG_AUTHENTICATED)
-                session->session_state = SSH_SESSION_STATE_AUTHENTICATED;
-
-        }
-        break;
-    case SSH_SESSION_STATE_AUTHENTICATING:
-        break;
-    case SSH_SESSION_STATE_ERROR:
-        goto error;
-    default:
-        ssh_set_error(session, SSH_FATAL, "Invalid state %d",
-                      session->session_state);
+            break;
+        case SSH_SESSION_STATE_AUTHENTICATING:
+            break;
+        case SSH_SESSION_STATE_ERROR:
+            goto error;
+        default:
+            ssh_set_error(session,SSH_FATAL,"Invalid state %d",session->session_state);
     }
 
     return;
 error:
     ssh_socket_close(session->socket);
     session->alive = 0;
-    session->session_state = SSH_SESSION_STATE_ERROR;
+    session->session_state=SSH_SESSION_STATE_ERROR;
 }
 
 /**
@@ -473,17 +458,16 @@ error:
  * @param  user is a pointer to session
  * @returns Number of bytes processed, or zero if the banner is not complete.
  */
-static int callback_receive_banner(const void *data, size_t len, void *user)
-{
-    char *buffer = (char *)data;
-    ssh_session session = (ssh_session)user;
+static size_t callback_receive_banner(const void *data, size_t len, void *user) {
+    char *buffer = (char *) data;
+    ssh_session session = (ssh_session) user;
     char *str = NULL;
     size_t i;
-    int ret=0;
+    size_t processed = 0;
 
     for (i = 0; i < len; i++) {
 #ifdef WITH_PCAP
-        if (session->pcap_ctx && buffer[i] == '\n') {
+        if(session->pcap_ctx && buffer[i] == '\n') {
             ssh_pcap_context_write(session->pcap_ctx,
                                    SSH_PCAP_DIR_IN,
                                    buffer,
@@ -492,34 +476,33 @@ static int callback_receive_banner(const void *data, size_t len, void *user)
         }
 #endif
         if (buffer[i] == '\r') {
-            buffer[i] = '\0';
+            buffer[i]='\0';
         }
 
         if (buffer[i] == '\n') {
-            buffer[i] = '\0';
+            buffer[i]='\0';
 
             str = strdup(buffer);
             /* number of bytes read */
-            ret = i + 1;
+            processed = i + 1;
             session->clientbanner = str;
             session->session_state = SSH_SESSION_STATE_BANNER_RECEIVED;
             SSH_LOG(SSH_LOG_PACKET, "Received banner: %s", str);
             session->ssh_connection_callback(session);
 
-            return ret;
+            return processed;
         }
 
-        if (i > 127) {
+        if(i > 127) {
             /* Too big banner */
             session->session_state = SSH_SESSION_STATE_ERROR;
-            ssh_set_error(session, SSH_FATAL,
-                          "Receiving banner: too large banner");
+            ssh_set_error(session, SSH_FATAL, "Receiving banner: too large banner");
 
             return 0;
         }
     }
 
-    return ret;
+    return processed;
 }
 
 /* returns 0 until the key exchange is not finished */
@@ -540,15 +523,35 @@ void ssh_set_auth_methods(ssh_session session, int auth_methods)
     session->auth.supported_methods = (uint32_t)auth_methods & 0x3fU;
 }
 
-/* Do the banner and key exchange */
-int ssh_handle_key_exchange(ssh_session session)
+int ssh_send_issue_banner(ssh_session session, const ssh_string banner)
 {
-    int rc;
+    int rc = SSH_ERROR;
 
-    if (session->session_state != SSH_SESSION_STATE_NONE) {
-        goto pending;
+    if (session == NULL) {
+        return SSH_ERROR;
     }
 
+    SSH_LOG(SSH_LOG_PACKET,
+            "Sending a server issue banner");
+
+    rc = ssh_buffer_pack(session->out_buffer,
+                         "bS",
+                         SSH2_MSG_USERAUTH_BANNER,
+                         banner);
+    if (rc != SSH_OK) {
+        ssh_set_error_oom(session);
+        return SSH_ERROR;
+    }
+
+    rc = ssh_packet_send(session);
+    return rc;
+}
+
+/* Do the banner and key exchange */
+int ssh_handle_key_exchange(ssh_session session) {
+    int rc;
+    if (session->session_state != SSH_SESSION_STATE_NONE)
+      goto pending;
     rc = ssh_send_banner(session, 1);
     if (rc < 0) {
         return SSH_ERROR;
@@ -559,28 +562,27 @@ int ssh_handle_key_exchange(ssh_session session)
     session->ssh_connection_callback = ssh_server_connection_callback;
     session->session_state = SSH_SESSION_STATE_SOCKET_CONNECTED;
     ssh_socket_set_callbacks(session->socket,&session->socket_callbacks);
-    session->socket_callbacks.data = callback_receive_banner;
-    session->socket_callbacks.exception = ssh_socket_exception_callback;
-    session->socket_callbacks.userdata = session;
+    session->socket_callbacks.data=callback_receive_banner;
+    session->socket_callbacks.exception=ssh_socket_exception_callback;
+    session->socket_callbacks.userdata=session;
 
     rc = server_set_kex(session);
     if (rc < 0) {
         return SSH_ERROR;
     }
-pending:
+    pending:
     rc = ssh_handle_packets_termination(session, SSH_TIMEOUT_USER,
-                                        ssh_server_kex_termination,session);
+        ssh_server_kex_termination,session);
     SSH_LOG(SSH_LOG_PACKET, "ssh_handle_key_exchange: current state : %d",
-            session->session_state);
-    if (rc != SSH_OK) {
-        return rc;
-    }
+        session->session_state);
+    if (rc != SSH_OK)
+      return rc;
     if (session->session_state == SSH_SESSION_STATE_ERROR ||
         session->session_state == SSH_SESSION_STATE_DISCONNECTED) {
-        return SSH_ERROR;
+      return SSH_ERROR;
     }
 
-    return SSH_OK;
+  return SSH_OK;
 }
 
 /* messages */
@@ -728,7 +730,7 @@ int ssh_message_global_request_reply_success(ssh_message msg, uint16_t bound_por
             goto error;
         }
 
-        if(msg->global_request.type == SSH_GLOBAL_REQUEST_TCPIP_FORWARD 
+        if(msg->global_request.type == SSH_GLOBAL_REQUEST_TCPIP_FORWARD
                                 && msg->global_request.bind_port == 0) {
             rc = ssh_buffer_pack(msg->session->out_buffer, "d", bound_port);
             if (rc != SSH_OK) {
@@ -740,7 +742,7 @@ int ssh_message_global_request_reply_success(ssh_message msg, uint16_t bound_por
         return ssh_packet_send(msg->session);
     }
 
-    if(msg->global_request.type == SSH_GLOBAL_REQUEST_TCPIP_FORWARD 
+    if(msg->global_request.type == SSH_GLOBAL_REQUEST_TCPIP_FORWARD
                                 && msg->global_request.bind_port == 0) {
         SSH_LOG(SSH_LOG_PACKET,
                 "The client doesn't want to know the remote port!");
@@ -1046,7 +1048,7 @@ int ssh_message_auth_reply_pk_ok_simple(ssh_message msg) {
     ssh_string pubkey_blob = NULL;
     int ret;
 
-    algo = ssh_string_from_char(msg->auth_request.pubkey->type_c);
+    algo = ssh_string_from_char(msg->auth_request.sigtype);
     if (algo == NULL) {
         return SSH_ERROR;
     }
